@@ -2,13 +2,15 @@
 /*
  * SPDX-License-Identifier: AGPL-3.0-only
  * SPDX-FileCopyrightText: Copyright 2016 - 2018 Kopano b.v.
- * SPDX-FileCopyrightText: Copyright 2020 grommunio GmbH
+ * SPDX-FileCopyrightText: Copyright 2020 - 2024 grommunio GmbH
  *
  * grommunio DAV backend class which handles grommunio related activities.
  */
 
 namespace grommunio\DAV;
 
+use Sabre\CalDAV\Xml\Property\SupportedCalendarComponentSet;
+
 class GrommunioDavBackend {
 	private $logger;
 	protected $session;
@@ -48,7 +50,7 @@ class GrommunioDavBackend {
 		$this->user = $user;
 		$this->logger->debug("Auth: OK - user %s - session %s", $this->user, $this->session);
 
-		return true;
+		return $this->isGdavEnabled();
 	}
 
 	/**
@@ -105,7 +107,6 @@ class GrommunioDavBackend {
 	 * Returns a list of folders for a MAPI class.
 	 *
 	 * @param string $principalUri
-	 * @param string $class
 	 * @param mixed  $classes
 	 *
 	 * @return array
@@ -116,7 +117,9 @@ class GrommunioDavBackend {
 
 		// TODO limit the output to subfolders of the principalUri?
 
-		$rootfolder = mapi_msgstore_openentry($this->GetStore($principalUri));
+		$store = $this->GetStore($principalUri);
+		$storeprops = mapi_getprops($store, [PR_IPM_WASTEBASKET_ENTRYID]);
+		$rootfolder = mapi_msgstore_openentry($store);
 		$hierarchy = mapi_folder_gethierarchytable($rootfolder, CONVENIENT_DEPTH | MAPI_DEFERRED_ERRORS);
 		// TODO also filter hidden folders
 		$restrictions = [];
@@ -126,7 +129,7 @@ class GrommunioDavBackend {
 		mapi_table_restrict($hierarchy, [RES_OR, $restrictions]);
 
 		// TODO how to handle hierarchies?
-		$rows = mapi_table_queryallrows($hierarchy, [PR_DISPLAY_NAME, PR_ENTRYID, PR_SOURCE_KEY, PR_PARENT_SOURCE_KEY, PR_FOLDER_TYPE, PR_LOCAL_COMMIT_TIME_MAX]);
+		$rows = mapi_table_queryallrows($hierarchy, [PR_DISPLAY_NAME, PR_ENTRYID, PR_SOURCE_KEY, PR_PARENT_SOURCE_KEY, PR_FOLDER_TYPE, PR_LOCAL_COMMIT_TIME_MAX, PR_CONTAINER_CLASS, PR_COMMENT, PR_PARENT_ENTRYID]);
 
 		$rootprops = mapi_getprops($rootfolder, [PR_IPM_CONTACT_ENTRYID, PR_IPM_APPOINTMENT_ENTRYID]);
 		foreach ($rows as $row) {
@@ -134,15 +137,28 @@ class GrommunioDavBackend {
 				continue;
 			}
 
+			if (isset($row[PR_PARENT_ENTRYID], $storeprops[PR_IPM_WASTEBASKET_ENTRYID]) && $row[PR_PARENT_ENTRYID] == $storeprops[PR_IPM_WASTEBASKET_ENTRYID]) {
+				continue;
+			}
+
 			$folder = [
 				'id' => $principalUri . ":" . bin2hex($row[PR_SOURCE_KEY]),
 				'uri' => $row[PR_DISPLAY_NAME],
 				'principaluri' => $principalUri,
 				'{http://sabredav.org/ns}sync-token' => '0000000000',
 				'{DAV:}displayname' => $row[PR_DISPLAY_NAME],
+				'{urn:ietf:params:xml:ns:caldav}calendar-description' => $row[PR_COMMENT],
 				'{http://calendarserver.org/ns/}getctag' => isset($row[PR_LOCAL_COMMIT_TIME_MAX]) ? strval($row[PR_LOCAL_COMMIT_TIME_MAX]) : '0000000000',
 			];
 
+			// set the supported component (task or calendar)
+			if ($row[PR_CONTAINER_CLASS] == "IPF.Task") {
+				$folder['{urn:ietf:params:xml:ns:caldav}supported-calendar-component-set'] = new SupportedCalendarComponentSet(['VTODO']);
+			}
+			if ($row[PR_CONTAINER_CLASS] == "IPF.Appointment") {
+				$folder['{urn:ietf:params:xml:ns:caldav}supported-calendar-component-set'] = new SupportedCalendarComponentSet(['VEVENT']);
+			}
+
 			// ensure default contacts folder is put first, some clients
 			// i.e. Apple Addressbook only supports one contact folder,
 			// therefore it is desired that folder is the default one.
@@ -164,27 +180,22 @@ class GrommunioDavBackend {
 	}
 
 	/**
-	 * Returns a list of objects for a folder given by the id.
+	 * Returns a MAPI restriction for a defined set of filters.
 	 *
-	 * @param string $id
-	 * @param string $fileExtension
 	 * @param array  $filters
+	 * @param string $storeId (optional) mapi compatible storeid - required when using start+end filter
 	 *
-	 * @return array
+	 * @return array|null
 	 */
-	public function GetObjects($id, $fileExtension, $filters = []) {
-		$folder = $this->GetMapiFolder($id);
-		$properties = $this->GetCustomProperties($id);
-		$table = mapi_folder_getcontentstable($folder, MAPI_DEFERRED_ERRORS);
-
+	private function getRestrictionForFilters($filters, $storeId = null) {
 		$restrictions = [];
-		if (isset($filters['start'], $filters['end'])) {
-			$this->logger->trace("got start: %d and end: %d", $filters['start'], $filters['end']);
-			$subrestriction = $this->GetCalendarRestriction($this->GetStoreById($id), $filters['start'], $filters['end']);
+		if (isset($filters['start'], $filters['end'], $storeId)) {
+			$this->logger->trace("getRestrictionForFilters - got start: %d and end: %d", $filters['start'], $filters['end']);
+			$subrestriction = $this->GetCalendarRestriction($storeId, $filters['start'], $filters['end']);
 			$restrictions[] = $subrestriction;
 		}
 		if (isset($filters['types'])) {
-			$this->logger->trace("got types: %s", $filters['types']);
+			$this->logger->trace("getRestrictionForFilters - got types: %s", $filters['types']);
 			$arr = [];
 			foreach ($filters['types'] as $filter) {
 				$arr[] = [RES_PROPERTY,
@@ -198,7 +209,29 @@ class GrommunioDavBackend {
 		}
 		if (!empty($restrictions)) {
 			$restriction = [RES_AND, $restrictions];
-			$this->logger->trace("Got restriction: %s", $restriction);
+			$this->logger->trace("getRestrictionForFilters - got restriction: %s", $restriction);
+
+			return $restriction;
+		}
+
+		return null;
+	}
+
+	/**
+	 * Returns a list of objects for a folder given by the id.
+	 *
+	 * @param string $id
+	 * @param string $fileExtension
+	 * @param array  $filters
+	 *
+	 * @return array
+	 */
+	public function GetObjects($id, $fileExtension, $filters = []) {
+		$folder = $this->GetMapiFolder($id);
+		$properties = $this->GetCustomProperties($id);
+		$table = mapi_folder_getcontentstable($folder, MAPI_DEFERRED_ERRORS);
+		$restriction = $this->getRestrictionForFilters($filters, $this->GetStoreById($id));
+		if ($restriction) {
 			mapi_table_restrict($table, $restriction);
 		}
 
@@ -238,10 +271,10 @@ class GrommunioDavBackend {
 	 * Create the object and set appttsref.
 	 *
 	 * @param mixed  $folderId
-	 * @param string $folder
+	 * @param mixed  $folder
 	 * @param string $objectId
 	 *
-	 * @return mapiresource
+	 * @return mixed
 	 */
 	public function CreateObject($folderId, $folder, $objectId) {
 		$mapimessage = mapi_folder_createmessage($folder);
@@ -259,7 +292,7 @@ class GrommunioDavBackend {
 	 *
 	 * @param string $folderid
 	 *
-	 * @return mapiresource
+	 * @return mixed
 	 */
 	public function GetMapiFolder($folderid) {
 		$this->logger->trace('Id: %s', $folderid);
@@ -272,7 +305,7 @@ class GrommunioDavBackend {
 	/**
 	 * Returns MAPI addressbook.
 	 *
-	 * @return MAPIAddressbook
+	 * @return mixed
 	 */
 	public function GetAddressBook() {
 		// TODO should be a singleton
@@ -284,7 +317,7 @@ class GrommunioDavBackend {
 	 *
 	 * @param string $username
 	 *
-	 * @return false|MAPIStore if store not available
+	 * @return mixed
 	 */
 	public function OpenMapiStore($username = null) {
 		$msgstorestable = mapi_getmsgstorestable($this->session);
@@ -324,7 +357,7 @@ class GrommunioDavBackend {
 	 *
 	 * @param string $storename
 	 *
-	 * @return false|MAPIStore if the store is not available
+	 * @return mixed
 	 */
 	public function GetStore($storename) {
 		if ($storename == null) {
@@ -355,7 +388,7 @@ class GrommunioDavBackend {
 	 *
 	 * @param mixed $id
 	 *
-	 * @return false|\grommunio\DAV\MAPIStore on error
+	 * @return mixed
 	 */
 	public function GetStoreById($id) {
 		$arr = explode(':', $id);
@@ -366,7 +399,7 @@ class GrommunioDavBackend {
 	/**
 	 * Returns logon session.
 	 *
-	 * @return MAPISession
+	 * @return mixed
 	 */
 	public function GetSession() {
 		return $this->session;
@@ -378,8 +411,8 @@ class GrommunioDavBackend {
 	 *
 	 * This order is reflected as well when searching for a message with these ids in GrommunioDavBackend->GetMapiMessageForId().
 	 *
-	 * @param string       $folderId
-	 * @param mapiresource $mapimessage
+	 * @param string $folderId
+	 * @param mixed  $mapimessage
 	 *
 	 * @return string
 	 */
@@ -410,12 +443,12 @@ class GrommunioDavBackend {
 	 * Finds and opens a MapiMessage from an objectId.
 	 * The id can be a PROP_APPTTSREF or a PR_SOURCE_KEY (as hex).
 	 *
-	 * @param string       $folderId
-	 * @param string       $objectUri
-	 * @param mapiresource $mapifolder optional
-	 * @param string       $extension  optional
+	 * @param string $folderId
+	 * @param string $objectUri
+	 * @param mixed  $mapifolder optional
+	 * @param string $extension  optional
 	 *
-	 * @return null|mapiresource
+	 * @return mixed
 	 */
 	public function GetMapiMessageForId($folderId, $objectUri, $mapifolder = null, $extension = null) {
 		$this->logger->trace("Searching for '%s' in '%s' (%s) (%s)", $objectUri, $folderId, $mapifolder, $extension);
@@ -493,7 +526,7 @@ class GrommunioDavBackend {
 			$this->logger->debug("Try to get entryid from appttsref");
 			$arr = explode(':', $folderId);
 			$sk = $this->syncstate->getSourcekey($arr[1], $id);
-			if ($sk != null) {
+			if ($sk !== null) {
 				$this->logger->debug("Found sourcekey from appttsref %s", $sk);
 				$entryid = mapi_msgstore_entryidfromsourcekey($this->GetStoreById($arr[0]), hex2bin($arr[1]), hex2bin($sk));
 			}
@@ -569,7 +602,10 @@ class GrommunioDavBackend {
 		if (!isset($this->customprops[$id])) {
 			$this->logger->trace("Fetching properties id:%s", $id);
 			$store = $this->GetStoreById($id);
-			$properties = getPropIdsFromStrings($store, ["goid" => "PT_BINARY:PSETID_Meeting:0x3", "vcarduid" => MapiProps::PROP_VCARDUID]);
+			$properties = getPropIdsFromStrings($store, [
+				"goid" => "PT_BINARY:PSETID_Meeting:" . PidLidGlobalObjectId,
+				"vcarduid" => MapiProps::PROP_VCARDUID,
+			]);
 			$this->customprops[$id] = $properties;
 		}
 
@@ -581,9 +617,9 @@ class GrommunioDavBackend {
 	 * return future calendar items (until $end), plus those since $start.
 	 * Origins: Z-Push.
 	 *
-	 * @param MAPIStore $store the MAPI store
-	 * @param long      $start Timestamp since when to include messages
-	 * @param long      $end   Ending timestamp
+	 * @param mixed $store the MAPI store
+	 * @param int   $start Timestamp since when to include messages
+	 * @param int   $end   Ending timestamp
 	 *
 	 * @return array
 	 */
@@ -677,10 +713,11 @@ class GrommunioDavBackend {
 	 * @param string $syncToken
 	 * @param string $fileExtension
 	 * @param int    $limit
+	 * @param array  $filters
 	 *
-	 * @return array
+	 * @return array|null
 	 */
-	public function Sync($folderId, $syncToken, $fileExtension, $limit = null) {
+	public function Sync($folderId, $syncToken, $fileExtension, $limit = null, $filters = []) {
 		$arr = explode(':', $folderId);
 		$phpwrapper = new PHPWrapper($this->GetStoreById($folderId), $this->logger, $this->GetCustomProperties($folderId), $fileExtension, $this->syncstate, $arr[1]);
 		$mapiimporter = mapi_wrap_importcontentschanges($phpwrapper);
@@ -699,7 +736,7 @@ class GrommunioDavBackend {
 		}
 		else {
 			$value = $this->syncstate->getState($arr[1], $syncToken);
-			if ($value == null) {
+			if ($value === null) {
 				$this->logger->error("Unable to get value from token: %s - folderId: %s", $syncToken, $folderId);
 
 				return null;
@@ -707,12 +744,15 @@ class GrommunioDavBackend {
 			mapi_stream_write($stream, hex2bin($value));
 		}
 
+		// force restriction of "types" to export only appointments or contacts
+		$restriction = $this->getRestrictionForFilters($filters);
+
 		// The last parameter in mapi_exportchanges_config is buffer size for mapi_exportchanges_synchronize - how many
 		// changes will be processed in its call. Setting it to MAX_SYNC_ITEMS won't export more items than is set in
 		// the config. If there are more changes than MAX_SYNC_ITEMS the client will eventually catch up and sync
 		// the rest on the subsequent sync request(s).
 		$bufferSize = ($limit !== null && $limit > 0) ? $limit : MAX_SYNC_ITEMS;
-		mapi_exportchanges_config($exporter, $stream, SYNC_NORMAL | SYNC_UNICODE, $mapiimporter, null, false, false, $bufferSize);
+		mapi_exportchanges_config($exporter, $stream, SYNC_NORMAL | SYNC_UNICODE, $mapiimporter, $restriction, false, false, $bufferSize);
 		$changesCount = mapi_exportchanges_getchangecount($exporter);
 		$this->logger->debug("Exporter found %d changes, buffer size for mapi_exportchanges_synchronize %d", $changesCount, $bufferSize);
 		while (is_array(mapi_exportchanges_synchronize($exporter))) {
@@ -753,4 +793,48 @@ class GrommunioDavBackend {
 
 		return $result;
 	}
+
+	/**
+	 * Returns an array of necessary properties to set with default values.
+	 *
+	 * @see MapiProps::GetDefault...Properties()
+	 *
+	 * @param mixed $id           storeid
+	 * @param mixed $mapimessage  mapi message to check
+	 * @param array $propList     array of mapped properties
+	 * @param array $defaultProps array of necessary properties with default values
+	 *
+	 * @return array
+	 */
+	public function GetPropsToSet($id, $mapimessage, $propList, $defaultProps) {
+		$propsToSet = [];
+		$store = $this->GetStoreById($id);
+		$propList = getPropIdsFromStrings($store, $propList);
+		$props = mapi_getprops($mapimessage);
+
+		foreach ($defaultProps as $prop => $value) {
+			if (!isset($props[$propList[$prop]])) {
+				$propsToSet[$propList[$prop]] = $value;
+			}
+		}
+
+		return $propsToSet;
+	}
+
+	/**
+	 * Checks whether the user is enabled for grommunio-dav.
+	 *
+	 * @return bool
+	 */
+	private function isGdavEnabled() {
+		$storeProps = mapi_getprops($this->GetStore($this->GetUser()), [PR_EC_ENABLED_FEATURES_L]);
+		if ($storeProps[PR_EC_ENABLED_FEATURES_L] & UP_DAV) {
+			$this->logger->debug("user %s is enabled for grommunio-dav", $this->user);
+
+			return true;
+		}
+		$this->logger->debug("user %s is disabled for grommunio-dav", $this->user);
+
+		return false;
+	}
 }
